'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = configureMocha;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _checkReporter = require('./checkReporter');

var _checkReporter2 = _interopRequireDefault(_checkReporter);

var _mocha = require('mocha');

var _mocha2 = _interopRequireDefault(_mocha);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var defaults = {
  reporterOptions: {}
};

function configureMocha() {
  var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

  // infinite stack traces
  Error.stackTraceLimit = Infinity;

  // init defauls
  _lodash2.default.defaults(options, defaults);

  // check reporter
  (0, _checkReporter2.default)(options.reporter);

  // init mocha
  var mocha = new _mocha2.default();

  // reporter
  mocha.reporter(options.reporter, options.reporterOptions);

  // colors
  mocha.useColors(options.colors);

  // inline-diffs
  mocha.useInlineDiffs(options.inlineDiffs);

  // slow <ms>
  mocha.suite.slow(options.slow);

  // timeout <ms>
  if (options.timeout === 0) {
    mocha.enableTimeouts(false);
  } else {
    mocha.suite.timeout(options.timeout);
  }

  // bail
  mocha.suite.bail(options.bail);

  // grep
  if (options.grep) {
    mocha.grep(new RegExp(options.grep));
  }

  // fgrep
  if (options.fgrep) {
    mocha.grep(options.fgrep);
  }

  // invert
  if (options.invert) {
    mocha.invert();
  }

  // check-leaks
  if (options.checkLeaks) {
    mocha.checkLeaks();
  }

  // full-trace
  if (options.fullTrace) {
    mocha.fullTrace();
  }

  // growl
  if (options.growl) {
    mocha.growl();
  }

  // async-only
  if (options.asyncOnly) {
    mocha.asyncOnly();
  }

  // delay
  if (options.delay) {
    mocha.delay();
  }

  // retries
  if (options.retries) {
    mocha.suite.retries(options.retries);
  }

  // interface
  mocha.ui(options.ui);

  return mocha;
}
module.exports = exports['default'];